--[[=============================================================================
    PROJECTOR Protocol Initialization Functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end


PROTOCOL_DECLARATIONS = {}

function ON_DRIVER_EARLY_INIT.projector_init()

end

function ON_DRIVER_INIT.projector_init()
	--LogTrace("ON_DRIVER_INIT.ProtocolDeclarations()")
	for k,v in pairs(PROTOCOL_DECLARATIONS) do
		if (PROTOCOL_DECLARATIONS[k] ~= nil and type(PROTOCOL_DECLARATIONS[k]) == "function") then
			PROTOCOL_DECLARATIONS[k]()
		end
	end	
end

function ON_DRIVER_LATEINIT.projector_init()
end

function PROTOCOL_DECLARATIONS.SetGlobals()
	LogTrace("PROTOCOL_DECLARATIONS.SetGlobals()")

	--[[
	TODO: Determine projector operating conditions to be exposed for Composer Programming.
		Most projectors will report status, i.e. Warm Up,Cool Down and Error, etc.
		Lamp usage may also be reported. The sample variables below are referenced in the
		DEV_MSG.POWER() function in the device_messages.lua file.
	--]]
	C4:AddVariable('Projector_Operating_Status',0,'STRING',false,false)	
	C4:AddVariable('Lamp_Hours',0,'STRING',false,false)
	gProjector_Operating_Status = ""
	gLamp_Hours = ""
end
	
function PROTOCOL_DECLARATIONS.CommandsTableInit_IR()
	LogTrace("PROTOCOL_DECLARATIONS.CommandsTableInit_IR()")

	CMDS_IR = {
		--index:  Proxy Command Name
		--value:  IR Code ID	 
		--Power
		["ON"]             = "",
		["OFF"]            = "",
		
		--Input Toggle
		["INPUT_TOGGLE"]="",
		["TV_VIDEO"] 		= "",
		
		--Menu
		["INFO"] = "",	--Display
		["GUIDE"] = "",
		["MENU"] = "",
		["CANCEL"] = "",
		["UP"] = "",
		["DOWN"] = "",
		["LEFT"] = "",
		["RIGHT"] = "",
		["ENTER"] = "",	
		["RECALL"]         = "",
		["PREV"]           = "",
		
		--Digits
		["NUMBER_0"]       = "", 	
		["NUMBER_1"]       = "",		
		["NUMBER_2"]       = "",	
		["NUMBER_3"]       = "",	
		["NUMBER_4"]       = "",	
		["NUMBER_5"]       = "",	
		["NUMBER_6"]       = "",	
		["NUMBER_7"]       = "",	
		["NUMBER_8"]       = "",	
		["NUMBER_9"]       = "",
		["STAR"]           = "",
		["DOT"]            = "",
		
		--Volume
		["MUTE_ON"]        = "",
		["MUTE_OFF"]       = "",
		["MUTE_TOGGLE"]    = "",		
		["PULSE_VOL_DOWN"] = "",
		["PULSE_VOL_UP"]   = "",		
		["SET_VOLUME_LEVEL"] = "",		
		
	}			
	
	CMDS_IR[PROJECTOR_PROXY_BINDINGID] = {}	
	
end

function PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()")

	CMDS = {
		--index:  Proxy Command Name
		--value:  Protocol Command Data
		
		--Power
		["ON"]             = tohex('11 00 00'),
		["OFF"]            = tohex('11 01 00'),
		
		--Input Toggle
		["INPUT_TOGGLE"]="",
		["TV_VIDEO"]		= "",
		
		--Menu
		["INFO"] = "",	--Display
		["GUIDE"] 		= tohex('02 04 0B'),
		["MENU"] 			= tohex('02 04 0F'),
		["CANCEL"] 		= tohex('02 04 13'),
		["UP"] 			= tohex('02 04 0B'),
		["DOWN"] 			= tohex('02 04 0C'),
		["LEFT"] 			= tohex('02 04 0D'),
		["RIGHT"] 		= tohex('02 04 0E'),
		["ENTER"] 		= tohex('02 04 15'),	
		["RECALL"]         = "",
		["PREV"]           = "",
		
		--Digits
		["NUMBER_0"]       = "", 	
		["NUMBER_1"]       = "",		
		["NUMBER_2"]       = "",	
		["NUMBER_3"]       = "",	
		["NUMBER_4"]       = "",	
		["NUMBER_5"]       = "",	
		["NUMBER_6"]       = "",	
		["NUMBER_7"]       = "",	
		["NUMBER_8"]       = "",	
		["NUMBER_9"]       = "",
		["STAR"]           = "",
		["DOT"]            = "",
		
		--Volume
		["MUTE_ON"]        	= tohex('14 00 01'),
		["MUTE_OFF"]       	= tohex('14 00 00'),
		--["MUTE_TOGGLE"]    = "",	
		["PULSE_VOL_DOWN"] 	= tohex('14 02 00'),
		["PULSE_VOL_UP"]   	= tohex('14 01 00'),		
		["SET_VOLUME_LEVEL"] = tohex('13 2A'),		
	
		--Status Query
		["GET_VOLUME_STATUS"]		= tohex('14 03'),
		["GET_MUTE_STATUS"]			= tohex('14 00'),
		["GET_POWER_STATUS"]		= tohex('11 00'),
		["GET_LAMP_HOURS"]			= tohex('15 01'),
		["GET_CURRENT_INPUT"]		= tohex('13 01'),
		["GET_OPERATING_TEMPERATURE"]	= tohex('15 03'),
	}
	
	CMDS[PROJECTOR_PROXY_BINDINGID] = {}
	
end

function PROTOCOL_DECLARATIONS.InputOutputTableInit()
	LogTrace("PROTOCOL_DECLARATIONS.InputOutputTableInit()")
	----------------------------------------- [*COMMAND/RESPONSE HELPER TABLES*] -----------------------------------------
	tInputCommandMap = {
		--index:  Connection Name
		--value:  Protocol Command Data	
		--["ADAPTER PORT"] = "33", 
		["Computer In"] 	= tohex('13 01 00'),
		["Computer 2"]    	= tohex('13 01 08'),
		["Video"] 		= tohex('13 01 05'),
		["S-Video"]       	= tohex('13 01 06'),
		["HDMI 1"] 		= tohex('13 01 03'),
		["HDMI 2"] 		= tohex('13 01 07'),
		["HDMI 3"] 		= tohex('13 01 09'),
		["HDMI/MHL 4"]		= tohex('13 01 0e'),
		["HDBaseT"] 		= tohex('13 01 0C'),
          ["DVI"]           	= tohex('13 01 0A'),
          ["Component"]     	= tohex('13 01 0B'),
          ["USB_C Display"] 	= tohex('13 01 0F'),		
	}
	
	--tInputResponseMap = ReverseTable(tInputCommandMap)	-- Reverses the tInputCommandMap table	
	tInputResponseMap = {
		--index:  Connection Name
		--value:  Protocol Command Data	
		--["ADAPTER PORT"] = "33", 
		[0] = "Computer In",
		[8] = "Computer 2",
		[5] = "Video",
		[6] = "S-Video",
		[3] = "HDMI 1",
		[7] = "HDMI 2",
		[9] = "HDMI 3",
		[14] = "HDMI/MHL 4",
		[12] = "HDBaseT",
          [10] = "DVI",
          [11] = "Component",
          [15] = "USB_C Display",		
	}
	
	tInputCommandMap_IR = { 
		--index:  Connection Name
		--value:  IR Code ID
		--["ADAPTER PORT"] = "51362",
		[""] = "",	
	}

	----------------------------------------- [*I/O HELPER TABLES*] -----------------------------------------
	tOutputConnMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Output Connection Name
		[0] = "Output",
	}

	tInputConnMapByID = {
		--index:  mod 1000 value of Input Connection id
		--[0] = {Name = "INPUT HDMI 1",BindingID = PROJECTOR_PROXY_BINDINGID,},
		--The first two entries below are for the antenna connections which are bound to the tuner proxy; the in values will need to be updated base on the connection ID.
		[0] = {Name = "Computer In",BindingID = PROJECTOR_PROXY_BINDINGID,},
		[1] = {Name = "Video",BindingID = PROJECTOR_PROXY_BINDINGID,},
		[10] = {Name = "HDMI 1",BindingID = PROJECTOR_PROXY_BINDINGID,},
		[11] = {Name = "HDMI 2",BindingID = PROJECTOR_PROXY_BINDINGID,},
		[12] = {Name = "HDMI 3",BindingID = PROJECTOR_PROXY_BINDINGID,},
		[20] = {Name = "HDBaseT",BindingID = PROJECTOR_PROXY_BINDINGID,},
	}
	
	tInputConnMapByName = {
		--index:  Input Connection Name
		--ID: mod 1000 value of Input Connection id
		--["INPUT HDMI 1"] = {ID = 0,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["Computer In"] = {ID = 0,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["Video"] = {ID = 1,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["HDMI 1"] = {ID = 10,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["HDMI 2"] = {ID = 11,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["HDMI 3"] = {ID = 12,BindingID = PROJECTOR_PROXY_BINDINGID,},
		["HDBaseT"] = {ID = 20,BindingID = PROJECTOR_PROXY_BINDINGID,},
	}

end	

function PROTOCOL_DECLARATIONS.PowerCommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.PowerCommandsTableInit_Serial()")
	
end

function ReverseTable(a)
	local b = {}
	for k,v in pairs(a) do b[v] = k end
	return b
end

