--[[=============================================================================
    Commands received from the Projector proxy (ReceivedFromProxy)

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Power Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: ON
	Parameters:
--]]

gCheckPowerStateFromOnCommand = false

function ON()
    local command_delay = tonumber(Properties["Power On Delay Seconds"])
    local delay_units = "SECONDS"
    gPowerOnTimer:StartTimer(command_delay, delay_units)
    
    gCheckPowerStateFromOnCommand = true
    GetDevicePowerStatus()
--[[
    if ((gProjectorProxy._PowerState == 'ON') and (gControlMethod ~= "IR")) then
	   gCheckPowerStateFromOnCommand = true
	   GetDevicePowerStatus()
	   return
    end

    local command_delay = tonumber(Properties["Power On Delay Seconds"])
    local delay_units = "SECONDS"
    
    if gPowerOffTimer:TimerStarted() then
	   --quick power on AFTER power off, need TV to turn off before sending on. This occurs in OnPowerOffTimer...
	   local off_time_remaining = tonumber(Properties["Power Off Delay Seconds"]) - (os.time() - gPowerOffStartTime)
	   LogInfo("Delaying Power ON until Power OFF sequence has finished! off_time_remaining: " .. off_time_remaining)
	   gPowerOnTimer:StartTimer(command_delay + off_time_remaining + 4, delay_units)
	   return
    else
	   gPowerOnTimer:StartTimer(command_delay, delay_units)
    end
        
    
    local command
    if (gControlMethod == "IR") then
		command = CMDS_IR["ON"]
		LogTrace("command = " .. command)
		PackAndQueueCommand("ON", command, command_delay, delay_units)
    else
    
		command = CMDS["ON"]
		--LogTrace("ON command:")
		--dbgHexdump(command)
		PackAndQueueCommand("ON", command, command_delay, delay_units)
		
    end
--]]
end

--[[
	Proxy Command: OFF
	Parameters:
--]]

gPowerOffStartTime = 0

function OFF()
    local command_delay = tonumber(Properties["Power Off Delay Seconds"])
    local delay_units = "SECONDS"
    
    --in case of a quick power on / power off sequence kill the power on timers in progress...
    gPowerOnTimer:KillTimer()
    gPowerOffTimer:StartTimer(tonumber(Properties["Power Off Delay Seconds"]), "SECONDS")
    gPowerOffStartTime = os.time()
    
    gQueryLampHoursDelayTimer:KillTimer()
    
    
    local command 
    if (gControlMethod == "IR") then
		-- TODO: create packet/command to send to the device
		command = CMDS_IR["OFF"]
		LogTrace("command = " .. command)
		PackAndQueueCommand("OFF", command, command_delay, delay_units)		
    else
		-- TODO: create packet/command to send to the device
		command = CMDS["OFF"]
		LogTrace("OFF command:")
		dbgHexdump(command)
		PackAndQueueCommand("OFF", command, command_delay, delay_units)	
    end  

end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Input Selection Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: SET_INPUT
	Parameters:
		idBinding: proxybindingid of proxy bound to input connection
		output: mod 1000 value of Output Connection id	
		input: mod 1000 value of Input Connection id
--]]
function SET_INPUT(idBinding, input)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = tInputCommandMap_IR[tInputConnMapByID[input].Name]	
	else
		-- TODO: create packet/command to send to the device
		--Edit the Input Selection command syntax based upon the protocol specification
		--if the tables referenced below are set up properly them no editing may be necessary 	
		command = tInputCommandMap[tInputConnMapByID[input].Name] 
	end 		

	LogTrace("SET_INPUT command:")
	dbgHexdump(command)
	--PackAndQueueCommand("SET_INPUT", command)
	
     local delay, delay_units
	if (gPowerOffTimer:TimerStarted()) then
	   delay = 120	--tonumber(Properties["Power On Delay Seconds"])
	   delay_units = "SECONDS"
     --elseif (gPowerOnTimer:TimerStarted()) then
	elseif (gProjector_Operating_Status ~= "ON") then
	   delay = 120	--tonumber(Properties["Power On Delay Seconds"])
	   delay_units = "SECONDS"
     else
	   --delay = tonumber(Properties["Command Delay Milliseconds"])
	   --delay_units = "MILLISECONDS"
	   delay = 1	
	   delay_units = "SECONDS"
     end
    
     --PackAndQueueCommand("EXIT", CMDS.CANCEL)  --Sending the Exit command before each input command to close OSD
    
     gSetInputDelayTimer:StartTimer(delay, delay_units)
     gSetInputDelayTimer._CallbackParam = command	   	
end

function OnSetInputDelayTimerExpired(command)
    LogTrace("OnSetInputDelayTimerExpired, command = " .. command)
    gSetInputDelayTimer:KillTimer()
    
    PackAndQueueCommand("SET_INPUT", command)
    
end

function OnQueryInputDelayTimerExpired()
    LogTrace("OnQueryInputDelayTimerExpired")
    gQueryInputDelayTimer:KillTimer()

    PackAndQueueReadCommand("CURRENT_INPUT", CMDS.GET_CURRENT_INPUT)
    
end


--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Volume Control Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: MUTE_OFF
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_OFF()
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR["MUTE_OFF"]	
	else
		-- TODO: create packet/command to send to the device
		command = CMDS["MUTE_OFF"]
	end 	
	LogTrace("MUTE_OFF command:")
	--dbgHexdump(command)
	PackAndQueueCommand("MUTE_OFF", command)
end

--[[
	Proxy Command: MUTE_ON
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_ON()
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR["MUTE_ON"]	
	else
		-- TODO: create packet/command to send to the device
		command = CMDS["MUTE_ON"]
	end 	
	LogTrace("MUTE_ON command:")
	--dbgHexdump(command)
	PackAndQueueCommand("MUTE_ON", command)		
end

--[[
	Proxy Command: MUTE_TOGGLE
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]

gMuteToggleInProgress = false
function MUTE_TOGGLE()
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR["MUTE_TOGGLE"]	
		PackAndQueueCommand("MUTE_TOGGLE", command)
	else
	     gMuteToggleInProgress = true
		PackAndQueueReadCommand("MUTE_STATUS", CMDS.GET_MUTE_STATUS)
	end 
end

--[[
	Proxy Command: SET_VOLUME_LEVEL
	Parameters:
		output: mod 1000 value of Output Connection id	
		c4VolumeLevel: volume level to be set represented in Control4 scale (0-100)
--]]
function SET_VOLUME_LEVEL(c4VolumeLevel)
     if (gMuteState == "True") then PackAndQueueCommand("MUTE_OFF", CMDS["MUTE_OFF"]) end
	
	local minDeviceLevel = MIN_DEVICE_LEVEL
	local maxDeviceLevel = MAX_DEVICE_LEVEL
	local deviceVolumeLevel = ConvertVolumeToDevice(c4VolumeLevel, minDeviceLevel, maxDeviceLevel)	
	deviceVolumeLevel = string.pack('b', deviceVolumeLevel)
	
	---- TODO: uncomment and edit string padding, if required, based upon the protocol specification
    --deviceVolumeLevel = string.rep("0", 2 - string.len(deviceVolumeLevel)) .. deviceVolumeLevel
	
	LogInfo('deviceVolumeLevel: ' .. deviceVolumeLevel)
  
	-- TODO: create packet/command to send to the device
	local command = CMDS["SET_VOLUME_LEVEL"] .. deviceVolumeLevel
	LogTrace("SET_VOLUME_LEVEL command:")
	dbgHexdump(command)
	PackAndQueueCommand("SET_VOLUME_LEVEL", command)
end

--[[
	Helper Function: SET_VOLUME_LEVEL_DEVICE
	Parameters:
		output: mod 1000 value of Output Connection id	
		deviceVolumeLevel: volume level to be set represented in device scale (as sepcified in the device's control protocol)
--]]
function SET_VOLUME_LEVEL_DEVICE(deviceVolumeLevel)
	--Called from ContinueVolumeRamping()
	
	-- TODO: create packet/command to send to the device
	deviceVolumeLevel = string.pack('b', deviceVolumeLevel)
	local command = CMDS["SET_VOLUME_LEVEL"] .. deviceVolumeLevel
	
	LogTrace("SET_VOLUME_LEVEL_DEVICE command:")
	dbgHexdump(command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("SET_VOLUME_LEVEL_DEVICE", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_DOWN
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_DOWN()
     if (gMuteState == "True") then PackAndQueueCommand("MUTE_OFF", CMDS["MUTE_OFF"]) end

	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR["PULSE_VOL_DOWN"]	
	else
		-- TODO: create packet/command to send to the device
		command = CMDS["PULSE_VOL_DOWN"]
	end 		
	LogTrace("PULSE_VOL_DOWN command")
	dbgHexdump(command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_DOWN", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_UP
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_UP()
     if (gMuteState == "True") then PackAndQueueCommand("MUTE_OFF", CMDS["MUTE_OFF"]) end
	
	local command
	if (gControlMethod == "IR") then	
		-- TODO: create packet/command to send to the device
		command = CMDS_IR["PULSE_VOL_UP"]	
	else
		-- TODO: create packet/command to send to the device
		command = CMDS["PULSE_VOL_UP"]
	end 		
	LogTrace("PULSE_VOL_UP command")
	dbgHexdump(command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_UP", command, command_delay)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Helper Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

--[[
	Helper Function: SEND_COMMAND_FROM_COMMAND_TABLE
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be sent
--]]
function SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, command_name)
    LogTrace("SEND_COMMAND_FROM_COMMAND_TABLE(), idBinding=" .. idBinding .. ", command_name=" .. command_name)
	
	-- TODO: create packet/command to send to the device
	local command = GetCommandFromCommandTable(idBinding, command_name)
	
	if (command == nil) then
		LogTrace("command is nil")
	else
		LogTrace("command = " .. command)
		PackAndQueueCommand(command_name, command)
	end			
end

--[[
	Helper Function: GetCommandFromCommandTable
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be returned
--]]
function GetCommandFromCommandTable(idBinding, command_name)
	LogTrace("GetCommand()")
	local t = {}
	
	-- TODO: create appropriate commands table structure
	
	if (gControlMethod == "IR") then
		t = CMDS_IR
	else
		t = CMDS
	end	

	if (t[idBinding] ~= nil) then
	   if (t[idBinding][command_name] ~= nil) then
		  return t[idBinding][command_name]
	   end
	end
	
	if (t[command_name] ~= nil) then
		return t[command_name]
	else
		LogWarn('GetCommandFromCommandTable: command not defined - '.. command_name)
		return nil
	end	
	
end

--[[
	Helper Function: GetDeviceVolumeStatus
--]]
function GetDeviceVolumeStatus()
    LogTrace("GetDeviceVolumeStatus()")
	
	PackAndQueueReadCommand("VOLUME_STATUS", CMDS["GET_VOLUME_STATUS"])
	
	PackAndQueueReadCommand("MUTE_STATUS", CMDS["GET_MUTE_STATUS"])	
end

--[[
	Helper Function: GetLampHours
--]]
function GetLampHours()
    LogTrace("GetLampHours()")
    PackAndQueueReadCommand("LAMP_HOURS", CMDS["GET_LAMP_HOURS"])
end

--[[
	Helper Function: GetDevicePowerStatus
--]]
function GetDevicePowerStatus(...)
    local bPriority = select(1, ...) or false
    LogTrace("GetDevicePowerStatus()")
	
    if (bPriority) then
	   PackAndQueueReadPriority1Command("POWER_STATUS", CMDS.GET_POWER_STATUS)
    else
	   PackAndQueueReadCommand("POWER_STATUS", CMDS.GET_POWER_STATUS)
    end
end
	